import { HttpService } from '@nestjs/axios';
import { Injectable } from '@nestjs/common';
import { firstValueFrom } from 'rxjs';

@Injectable()
export class SupersetService {
  private SUPERSET_URL = 'https://dashboard.weplayground.eigen.co.id';
  constructor(private readonly httpService: HttpService) {}
  async getLoginToken() {
    const data = {
      username: 'admin',
      password: 'admin',
      provider: 'db',
      refresh: true,
    };
    const response = await firstValueFrom(
      this.httpService.post(`${this.SUPERSET_URL}/api/v1/security/login`, data),
    );

    return response.data.access_token;
  }

  async getCSRFToken() {
    const loginToken = await this.getLoginToken();

    const response = await firstValueFrom(
      this.httpService.get(`${this.SUPERSET_URL}/api/v1/security/csrf_token/`, {
        headers: { Authorization: `Bearer ${loginToken}` },
      }),
    );

    return { loginToken, csrfToken: response.data.result };
  }

  async getGuestToken(uuid: string) {
    const { loginToken, csrfToken } = await this.getCSRFToken();
    const data = {
      resources: [{ type: 'dashboard', id: uuid }],
      rls: [],
      user: { username: 'pos_embed', first_name: 'pos', last_name: 'embed' },
    };

    const headers = {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${loginToken}`,
      'X-CSRFToken': `${csrfToken}`,
      Referer: `${this.SUPERSET_URL}/api/v1/security/guest_token/`,
    };

    const response = await firstValueFrom(
      this.httpService.post(
        `${this.SUPERSET_URL}/api/v1/security/guest_token/`,
        data,
        { headers, xsrfHeaderName: 'X-CSRFToken' },
      ),
    );

    return response.data.token;
  }
}
